@@ -27,13 +27,9 @@
 #include <gio/gio.h>
 
 #include <gdk/gdk.h>
-#include <gdk/gdkx.h>
-#include <X11/Xlib.h>
-#include <X11/Xatom.h>
 #include <gtk/gtk.h>
 
 #include <cairo.h>
-#include <cairo-xlib.h>
 
 #define GNOME_DESKTOP_USE_UNSTABLE_API
 #include "gnome-bg.h"
@@ -302,16 +298,7 @@ gnome_bg_crossfade_set_start_surface (GnomeBGCrossfade *fade,
 static gdouble
 get_current_time (void)
 {
-	const double microseconds_per_second = (double) G_USEC_PER_SEC;
-	double timestamp;
-	GTimeVal now;
-
-	g_get_current_time (&now);
-
-	timestamp = ((microseconds_per_second * now.tv_sec) + now.tv_usec) /
-	            microseconds_per_second;
-
-	return timestamp;
+        return (double) g_get_real_time () / 1000000.0;
 }
 
 /**
@@ -365,34 +352,11 @@ animations_are_disabled (GnomeBGCrossfade *fade)
 	return !are_enabled;
 }
 
-static void
-send_root_property_change_notification (GnomeBGCrossfade *fade)
-{
-        long zero_length_pixmap;
-
-        /* We do a zero length append to force a change notification,
-         * without changing the value */
-        XChangeProperty (GDK_WINDOW_XDISPLAY (fade->priv->window),
-                         GDK_WINDOW_XID (fade->priv->window),
-                         gdk_x11_get_xatom_by_name ("_XROOTPMAP_ID"),
-                         XA_PIXMAP, 32, PropModeAppend,
-                         (guchar *) &zero_length_pixmap, 0);
-}
-
 static void
 draw_background (GnomeBGCrossfade *fade)
 {
 	if (gdk_window_get_window_type (fade->priv->window) == GDK_WINDOW_ROOT) {
-                XClearArea (GDK_WINDOW_XDISPLAY (fade->priv->window),
-                            GDK_WINDOW_XID (fade->priv->window),
-                            0, 0,
-                            gdk_window_get_width (fade->priv->window),
-                            gdk_window_get_height (fade->priv->window),
-                            False);
-
-                send_root_property_change_notification (fade);
-
-		gdk_flush ();
+		g_warning ("Crossfade is not supported on ROOT window!");
 	} else {
 		gdk_window_invalidate_rect (fade->priv->window, NULL, FALSE);
 		gdk_window_process_updates (fade->priv->window, FALSE);
