@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
@@ -37,9 +37,7 @@ int
 main (int argc, char *argv[])
 {
 	GnomePnpIds *ids;
-	guint i;
-
-	g_type_init ();
+	int i;
 
 	ids = gnome_pnp_ids_new ();
 
@@ -71,6 +69,7 @@ main (int argc, char *argv[])
 	if (argc < 2) {
 		show_vendor (ids, "ELO");
 		show_vendor (ids, "IBM");
+		show_vendor (ids, "DEL4085");
 	}
 
 	g_object_unref (ids);
