@@ -17,6 +17,8 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
+#include "config.h"
+
 #include <gdesktop-enums.h>
 #include <glib.h>
 #define GNOME_DESKTOP_USE_UNSTABLE_API
@@ -36,8 +38,7 @@ test_utf8_character (const char *utf8_char,
 {
 	GDateTime  *datetime;
 	GnomeWallClock *clock;
-	locale_t locale;
-	locale_t save_locale;
+	const char *save_locale;
 	const char *str;
 
 	/* When testing that UTF8 locales don't use double spaces
@@ -47,9 +48,7 @@ test_utf8_character (const char *utf8_char,
 
 	/* In the C locale, make sure the time string is formatted with regular
          * colons */
-	locale = newlocale (LC_ALL_MASK, "C", (locale_t) 0);
-	g_assert_true (locale != (locale_t)0);
-	save_locale = uselocale (locale);
+	save_locale = setlocale (LC_ALL, NULL);
 	clock = gnome_wall_clock_new ();
 	str = gnome_wall_clock_string_for_datetime (clock,
 	                                            datetime,
@@ -60,12 +59,7 @@ test_utf8_character (const char *utf8_char,
 	g_object_unref (clock);
 
 	/* In a UTF8 locale, we want ratio characters and no colons. */
-	locale = newlocale (LC_ALL_MASK, "en_US.utf8", locale);
-	if (locale == (locale_t)0) {
-		g_test_skip("en_US.utf8 locale not found, skipping");
-		return;
-	}
-	uselocale (locale);
+	setlocale (LC_ALL, "en_US.UTF-8");
 	clock = gnome_wall_clock_new ();
 	str = gnome_wall_clock_string_for_datetime (clock,
 	                                            datetime,
@@ -77,12 +71,7 @@ test_utf8_character (const char *utf8_char,
 
 	/* ... and same thing with an RTL locale: should be formatted with
          * ratio characters */
-	locale = newlocale (LC_ALL_MASK, "he_IL.utf8", locale);
-	if (locale == (locale_t)0) {
-		g_test_skip("he_IL.utf8 locale not found, skipping");
-		return;
-	}
-	uselocale (locale);
+	setlocale (LC_ALL, "he_IL.UTF-8");
 	clock = gnome_wall_clock_new ();
 	str = gnome_wall_clock_string_for_datetime (clock,
 	                                            datetime,
@@ -95,8 +84,7 @@ test_utf8_character (const char *utf8_char,
 	g_date_time_unref (datetime);
 
 	/* Restore previous locale */
-	uselocale (save_locale);
-	freelocale (locale);
+	setlocale (LC_ALL, save_locale);
 }
 
 static void
@@ -116,16 +104,11 @@ test_clock_format_setting (void)
 {
 	GnomeWallClock *clock;
 	GSettings *settings;
-	locale_t locale;
-	locale_t save_locale;
+	const char *save_locale;
 	const char *str;
 
-	locale = newlocale (LC_ALL_MASK, "en_US.utf8", (locale_t) 0);
-	if (locale == (locale_t)0) {
-		g_test_skip("en_US.utf8 locale not found, skipping");
-		return;
-	}
-	save_locale = uselocale (locale);
+	save_locale = setlocale (LC_ALL, NULL);
+	setlocale (LC_ALL, "en_US.UTF-8");
 
 	settings = g_settings_new ("org.gnome.desktop.interface");
 
@@ -146,8 +129,7 @@ test_clock_format_setting (void)
 	g_object_unref (settings);
 
 	/* Restore previous locale */
-	uselocale (save_locale);
-	freelocale (locale);
+	setlocale (LC_ALL, save_locale);
 }
 
 static gboolean
@@ -199,14 +181,12 @@ test_weekday_setting (void)
 {
 	GnomeWallClock *clock;
 	GSettings *settings;
-	locale_t locale;
-	locale_t save_locale;
+	const char *save_locale;
 	const char *str, *ptr, *s;
 
 	/* Save current locale */
-	locale = newlocale (LC_ALL_MASK, "C", (locale_t) 0);
-	g_assert_true (locale != (locale_t)0);
-	save_locale = uselocale (locale);
+	save_locale = setlocale (LC_ALL, NULL);
+	setlocale (LC_ALL, "C");
 	settings = g_settings_new ("org.gnome.desktop.interface");
 
 	/* Set 24h format, so that the only alphabetical part will be the weekday */
@@ -242,8 +222,7 @@ test_weekday_setting (void)
 	g_object_unref (settings);
 
 	/* Restore previous locale */
-	uselocale (save_locale);
-	freelocale (locale);
+	setlocale (LC_ALL, save_locale);
 }
 
 int
